<?php
 
require_once('Fivecms.php');

class Money extends Fivecms
{
	private $currencies = array();
	private $currency;

	public function __construct()
	{
		parent::__construct();
		
		$this->design->smarty->registerPlugin('modifier', 'convert', array($this, 'convert'));
		$this->design->smarty->registerPlugin('modifier', 'noformat', array($this, 'noFormat'));

		$this->init_currencies();
	}
	
	private function init_currencies()
	{
		$this->currencies = array();
		// Выбираем из базы валюты
		$query = "SELECT * FROM __currencies ORDER BY position";
		//$this->db->query($query);
		//$results = $this->db->results();
		
		if($this->settings->cached == 1 && empty($_SESSION['admin'])){
			if($results = $this->cache->get($query)) {
				// возвращаем данные из memcached
			} else {
				$this->db->query($query); // иначе тянем из БД
				$results = $this->db->results();
				$this->cache->set($query, $results); //помещаем в кеш
			}
		} else {
			$this->db->query($query);
			$results = $this->db->results();
		}
		
		foreach($results as $c)
		{
			$this->currencies[$c->id] = $c;
		}
		
		$this->currency = reset($this->currencies);
	}

	public function get_currencies($filter = array())
	{
		$currencies = array();
		foreach($this->currencies as $id=>$currency)
			if((isset($filter['enabled']) && $filter['enabled'] == 1 && $currency->enabled) || empty($filter['enabled']))
				$currencies[$id] = $currency;
			
		return $currencies;
	}
	
	public function get_currency($id = null)
	{
		if(!empty($id) && is_integer($id) && isset($this->currencies[$id]))
			return $this->currencies[$id];
			
		if(!empty($id) && is_string($id))
		{
			foreach($this->currencies as $currency)
			{
				$currency_code = mb_strtolower($currency->code);
				if($currency_code == $id)
					return $currency;
			}
		}

		return $this->currency;
	}
	
	public function add_currency($currency)
	{	
		$query = $this->db->placehold('INSERT INTO __currencies
		SET ?%',
		$currency);

		if(!$this->db->query($query))
			return false;

		$id = $this->db->insert_id();
		$this->db->query("UPDATE __currencies SET position=id WHERE id=?", $id);
		$this->init_currencies();
			
		return $id;
	}
	
	public function update_currency($id, $currency)
	{	
		$query = $this->db->placehold('UPDATE __currencies
						SET ?%
						WHERE id in (?@)',
					$currency, (array)$id);
		if(!$this->db->query($query))
			return false;
		
		$this->init_currencies();
		return $id;
	}
	
	public function delete_currency($id)
	{
		if(!empty($id))
		{
			$query = $this->db->placehold("DELETE FROM __currencies WHERE id=? LIMIT 1", intval($id));
			$this->db->query($query);
		}
		$this->init_currencies();		
	}	
	
	public function convert($price, $currency_id = null, $format = true)
	{
		if(isset($currency_id))
		{
			if(is_numeric($currency_id))
				$currency = $this->get_currency((integer)$currency_id);
			else
				$currency = $this->get_currency((string)$currency_id);
		}
		elseif(isset($_SESSION['currency_id']))
			$currency = $this->get_currency($_SESSION['currency_id']);
		else
			$currency = current($this->get_currencies(array('enabled'=>1)));
								
		$result = floatval($price);
		
		if(!empty($currency))
		{		
			// Умножим на курс валюты
			$result = $result*$currency->rate_from/$currency->rate_to;
			
			// Точность отображения, знаков после запятой
			$precision = isset($currency->cents)?$currency->cents:2;
		}
		
		// Форматирование цены
		if($format)
			$result = number_format($result, $precision, $this->settings->decimals_point, $this->settings->thousands_separator);
		else
			$result = round($result, $precision);

		return $result;
	}

	public function noFormat($price, $convert_type = null, $currency_id = null)
	{
		if(isset($currency_id))
		{
			if(is_numeric($currency_id))
				$currency = $this->get_currency((integer)$currency_id);
			else
				$currency = $this->get_currency((string)$currency_id);
		}
		elseif(isset($_SESSION['currency_id']))
			$currency = $this->get_currency($_SESSION['currency_id']);
		else{
			$get_currencies = $this->get_currencies(array('enabled'=>1));
			$currency = reset($get_currencies);
		}
								
		$result = floatval($price);
		
		if(!empty($currency))
		{	
			if($convert_type == 'deconvert'){
				$result = $result/$currency->rate_from*$currency->rate_to;
			} else {
				$result = $result*$currency->rate_from/$currency->rate_to;
			}
			// Точность отображения, знаков после запятой
			$precision = isset($currency->cents)?$currency->cents:2;
		}
		
		if($convert_type != 'no_precision')
			$result = round($result, $precision);

		return $result;
	}
	
	//Multicurrency
	//возвращаем  текущую валюту
    public function get_current()
    {
        // Берем валюту из сессии
        if(isset($_SESSION['currency_id']))
            $currency = $this->get_currency($_SESSION['currency_id']);
        // Или первую из списка
        else{
        	$get_currencies = $this->get_currencies(array('enabled'=>1));
            $currency = reset($get_currencies);
        }    
        return $currency;
    }
	//Multicurrency end

}
