<?php

require_once 'api/Fivecms.php';
require_once 'RBS.php';

class Sberbank extends Fivecms
{
    private $payment_method = [],
        $order = [],
        $payment_settings = [],
        $debug = 0;

    public function checkout_form($order_id, $button_text = null)
    {
        if (empty($button_text)) {
            $button_text = 'Перейти к оплате';
        }

        $this->order = $this->orders->get_order((int)$order_id);
        $this->payment_method = $this->payment->get_payment_method($this->order->payment_method_id);
        $this->payment_settings = $this->payment->get_payment_settings($this->payment_method->id);
        
        $total_price = round($this->money->convert($this->order->total_price, $this->payment_method->currency_id, false), 2);

        // Debug mode
        $this->payment_settings['sbr_debug'] == 1 AND $_SESSION['admin'] ? $this->debug = 1 : $this->debug = 0;

		if(isset($_SESSION['order'][$this->order->id]))
        	$sber_session = $_SESSION['order'][$this->order->id];
        $return_url = $this->config->root_url . "/payment/Sberbank/callback.php?order=" . $this->order->id;
        $order_description = 'Оплата заказа №' . $this->order->id . ' на сайте ' . $this->settings->site_name;
        $order_description = (string)mb_substr($order_description, 0, 99);


        /**
         * Подключаемся к эквайрингу
         */
        $rbs = new RBS($this->payment_settings['sbr_login'], $this->payment_settings['sbr_password'], FALSE, $this->payment_settings['sbr_mode'] ? TRUE : FALSE);

		if (isset($sber_session['order_prefix'])) {
            if ($sber_session['order_prefix'] < strtotime(date('Y-m-d H:i:s', strtotime('-20 min')))) {
                $order_id_store = $this->order->id;
                unset($_SESSION['order'][$this->order->id]);
                unset($sber_session);
            } else {
                $order_id_store = $this->order->id . '-' . $sber_session['order_prefix'];
            }
        } else {
            $order_id_store = $this->order->id;
        }

        // Узнаем статус заказа у Сбербанка
        $order_status = $rbs->get_order_status_by_orderNumber($order_id_store);

        /**
         * Подготавливаем корзину для 54-ФЗ
         */
        $orderBundle = [];
        
        // Сбрасываем данные если получаем ошибку с кодом 6
        $reset_bundle = 0;
        if(isset($order_status['errorCode']) && $order_status['errorCode'] == '6'){
        	$reset_bundle = 1;
        }

        if ($this->payment_settings['sbr_orderBundle'] && empty($reset_bundle)) {

            // Информация о пользователе
            $orderBundle['customerDetails'] = [
                "email" => $this->order->email,
                "phone" => preg_replace('/[^0-9]/', '', $this->order->phone)
            ];

            // Добавляем товары в чек
            $purchases_54 = $this->normalize($this->orders->get_purchases(['order_id' => $this->order->id]));
            foreach ($purchases_54 as $key => $purchase) {
            	
            	if(!empty($purchase->unit))
            		$unit = $purchase->unit;
            	else
            		$unit = $this->settings->units;
            		
                $orderBundle['cartItems']['items'][$key] = [
                    "positionId" => $key + 1,
                    "name" => mb_substr(html_entity_decode($purchase->product_name), 0, 64),
                    "quantity" => [
                        "value" => $purchase->amount,
                        "measure" => $unit
                    ],
                    "itemAmount" => $this->convert_price($purchase->price * $purchase->amount),
                    "itemCode" => $purchase->variant_id,
                    "tax" => [
                        "taxType" => isset($purchase->taxType) ? $purchase->taxType : $this->payment_settings['sbr_taxType']
                    ],
                    "itemPrice" => $this->convert_price($purchase->price),

                    // ФФД 1.05
                    "itemAttributes" => [
                        'attributes' => [
                            [
                                'name' => 'paymentMethod',
                                'value' => $this->payment_settings['sbr_paymentMethod']
                            ],
                            [
                                'name' => 'paymentObject',
                                'value' => $this->payment_settings['sbr_paymentObject']
                            ]
                        ]
                    ]
                ];
            }

            // Доставка
            if ($this->order->delivery_id && $this->order->delivery_price > 0 && !$this->order->separate_delivery) {
                $delivery = $this->delivery->get_delivery($this->order->delivery_id);

                	// Добавляем доставку в чек
                    $key = count($orderBundle['cartItems']['items']);
                    $orderBundle['cartItems']['items'][$key] = [
                        "positionId" => count($orderBundle['cartItems']['items']) + 1,
                        "name" => $delivery->name,
                        "quantity" => [
                            "value" => 1,
                            "measure" => 'шт'
                        ],
                        "itemAmount" => $this->convert_price($this->order->delivery_price),
                        "itemCode" => 'DELIVERY-' . $delivery->id,
                        "tax" => [
                            "taxType" => $this->payment_settings['sbr_taxType']
                        ],
                        "itemPrice" => $this->convert_price($this->order->delivery_price)
                    ];
            }

            $orderBundle = json_encode($orderBundle);
        }


        /**
         * Может мы пытались перерегестрировать заказ?
         */
        /*if (isset($sber_session['order_prefix'])) {
            if ($sber_session['order_prefix'] < strtotime(date('Y-m-d H:i:s', strtotime('-20 min')))) {
                $order_id_store = $this->order->id;
                unset($_SESSION['order'][$this->order->id]);
                unset($sber_session);
            } else {
                $order_id_store = $this->order->id . '-' . $sber_session['order_prefix'];
            }
        } else {
            $order_id_store = $this->order->id;
        }

        // Узнаем статус заказа у Сбербанка
        $order_status = $rbs->get_order_status_by_orderNumber($order_id_store);*/

        if ($this->debug) {
            print '<pre>';
            echo '<h1>Текущий заказ:</h1>';
            var_dump($this->order);
            echo '<h1>Сумма заказа для Сбера:</h1>';
            var_dump($this->convert_price($total_price));
            echo '<h1>Заказ в сессии:</h1>';
            var_dump($_SESSION['order']);
            echo '<h1>ID заказа:</h1>';
            var_dump($order_id_store);
            echo '<h1>Статус заказа от сбера:</h1>';
            var_dump($order_status);
            echo '<h1>Корзина для ФЗ-54:</h1>';
            var_dump(json_decode($orderBundle));
            print '</pre>';
        }

        /**
         * Истек срок ожидания ввода данных (заказ делали больше 20 минут назад)
         * поступила команда пересоздать заказ
         */
        if ($this->request->post('reorder', 'integer') == 1) {

            $order_prefix = strtotime(date('Y/m/d H:i:s'));

            // Регестрируем заказ с новым ID заказа (ID + order_prefix)
            $response = $rbs->register_order(
                $this->order->id . '-' . $order_prefix,
                $this->convert_price($total_price),
                $return_url,
                $order_description,
                $orderBundle,
                $this->payment_settings['sbr_taxSystem']
            );

            // Запомним новый номер заказа.
            $_SESSION['order'][$this->order->id] = [
                'orderId' => $response['orderId'],
                'formUrl' => $response['formUrl'],
                'order_prefix' => $order_prefix
            ];

            header('Location: ' . $response['formUrl']);
        }

        /**
         * Создаем новый заказ в ПШ
         */
        if (!isset($order_status['actionCode'])) {

            // Заказ не был создан, создаем.
            $response = $rbs->register_order(
                $this->order->id,
                $this->convert_price($total_price),
                $return_url,
                $order_description,
                $orderBundle,
                $this->payment_settings['sbr_taxSystem']
            );

			if(empty($reset_bundle)){
				if (!$response['errorCode']) {
					// Запомним новый номер заказа.
					$_SESSION['order'][$this->order->id] = [
						'orderId' => $response['orderId'],
						'formUrl' => $response['formUrl']
					];
					return "<a href='" . $response['formUrl'] . "' class='checkout_button'>" . $button_text . ' </a>';
				} elseif ($response == NULL) {
					return 'Невозможно подключиться к платежному шлюзу';
				} else {
					return $response['errorMessage'];
				}
				
			} else {
				// После сброса данных нужно заново войти на страницу заказа
				return "<a href='" . $this->config->root_url . '/order/' . $this->order->url . "' class='checkout_button'>Обновить страницу</a>";
			}		

        } elseif (isset($order_status['actionCode']) AND $order_status['actionCode'] != 0) {

            // Заказ был создан, но не не был оплачен, можно продолжить оплату
            // или нет префикса для продолжения заказа
            if (!isset($sber_session['formUrl']) OR $order_status['actionCode'] != '-100') {

                // Запрос отмены старого заказа
                $reverse_order_id = isset($sber_session['order_prefix']) ? $this->order->id . '-' . $sber_session['order_prefix'] : $this->order->id;
                $response_old_order = $rbs->reverse_order($reverse_order_id);
                // unset($_SESSION['order'][$this->order->id]);

                $button = '<p>Заказ не был оплачен.</p>' .
                    '<p>' . $order_status['actionCodeDescription'] . '</p>' .
                    '<form action="' . $this->config->root_url . '/order/' . $this->order->url . '" method=POST>' .
                    '<input type=hidden name=reorder value="1">' .
                    '<input type=submit class=checkout_button value="Повторить оплату">' .
                    '</form>';

            } else {
                $button = '<p>Заказ не был оплачен.</p>' .
                    '<p>' . $order_status['actionCodeDescription'] . '</p>' .
                    "<a href='" . $_SESSION['order'][$this->order->id]['formUrl'] . "' class='checkout_button'>" . $button_text . ' </a>';
            }

            return $button;

        } else {
            // Заказ был оплачен
            return '<p>Заказ был оплачен.</p>';
        }
    }


    /**
     * Конвертируем цену из 100.00 в 10000 для Платежного Шлюза
     * @param $price
     * @return integer
     */
    private function convert_price($price)
    {
        //return $result = round($price, 2) * 100;
        $result = round($price, 2) * 100;
        return (int)$result;
    }


    /**
     * Подгоняет стоимость товаров в чеке, кроме доставки, к общей цене заказа
     * @param object $purchases товары в заказе
     * @return object $purchases
     */
    private function normalize($purchases)
    {
        $final_purchases = [];

        /**
         * Подгоняем цены у товаров
         */
        $positions = [];
        
        // new_discount 
        // count discount
        $total_price = round($this->money->convert($this->order->total_price, $this->payment_method->currency_id, false), 2);
        
        if(!$this->order->separate_delivery)
			$total_price -= (float)$this->order->delivery_price; //Скидка не применяется к доставке
		$full_total_price = 0;
		foreach ($purchases as $p) {
			$full_total_price += (float)$p->price * (int)$p->amount;
		}
		if(!empty($full_total_price))
			$discount = $total_price / $full_total_price;
		else	
			$discount = 1;
					
		function endKey($array){
			end($array);
			return key($array);
		}		
        // count discount@
        
        foreach ($purchases as $key => $purchase) {
            if($key == endKey($purchases)) {
				$purchase_price = $total_price/$purchase->amount;
			} else {
				$purchase_price = round(((float)$purchase->price * $discount), 2);
				$total_price -= $purchase_price*$purchase->amount;
			}
                   
            $purchase->price = $purchase_price;
            $positions[] = $purchase;   
        }
        // new_discount @

        if ($this->debug) {
            print '<pre>';
            var_dump($positions);
            print '</pre>';
        }

        $purchases = $positions;

        /**
         * Кастомный НДС
         * для каждого товара
         */
        if (!empty($this->payment_method->sbr_taxProduct)) {
            $products_ids = [];
            foreach ($purchases as $item) {
                $products_ids[] = $item->product_id;
            }
            $products = [];
            foreach ($this->products->get_products(['id' => $products_ids]) as $product) {
                $products[$product->id] = $product;
            }
            foreach ($purchases as $item) {
                if (isset($products[$item->product_id]->taxType)) {
                    $item->taxType = $products[$item->product_id]->taxType;
                }
            }
        }

        return $purchases;
    }
}